package com.vmware.psc.alerting.persister.integration.data;

import static com.carbonblack.protocols.psc.response.PSCRAugmentedEventBatch.CbDeviceOsType.WINDOWS;
import static com.carbonblack.protocols.psc.response.PSCRAugmentedEventBatch.CbDeviceTargetPriority.TARGET_PRIORITY_HIGH;
import static com.carbonblack.protocols.psc.response.PSCRAugmentedEventBatch.CbDeviceTargetPriority.TARGET_PRIORITY_MEDIUM;

import com.carbonblack.protocols.psc.response.PSCRAugmentedEventBatch;
import java.io.IOException;
import java.util.Locale;
import net.datafaker.Faker;
import org.jetbrains.annotations.NotNull;
import org.springframework.core.io.ClassPathResource;

public class TestData {

  private static final Faker faker = new Faker();

  static {
    try {
      // Manually register the adjective file as it's not configured in datafaker :facepalm:
      var adj = new ClassPathResource("en/adjective.yml");
      faker.addUrl(Locale.ENGLISH, adj.getURL());
    } catch (IOException e) {
      throw new IllegalStateException(e);
    }
  }

  @NotNull
  public static String generateTestRunId() {
    return faker
        .expression("#{adjective.positive}-#{animal.name}-#{numerify '###'}")
        .replace(" ", "-")
        .toLowerCase();
  }

  @NotNull
  public static String generateTestDeviceName() {
    return faker
        .expression("#{hacker.adjective}-#{hacker.noun}-#{numerify '###'}")
        .replace(" ", "-")
        .toLowerCase();
  }

  @NotNull
  public static PSCRAugmentedEventBatch.CbDeviceMembership generateTestPolicy() {
    return PSCRAugmentedEventBatch.CbDeviceMembership.newBuilder()
        .setId(faker.number().positive())
        .setName(faker.science().element())
        .build();
  }

  @NotNull
  public static PSCRAugmentedEventBatch.CbDeviceMetadata.Builder generateTestDevice(
      @NotNull String orgKey,
      @NotNull String deviceName,
      @NotNull PSCRAugmentedEventBatch.CbDeviceMembership policy,
      @NotNull PSCRAugmentedEventBatch.CbDeviceOsType os) {
    var device =
        PSCRAugmentedEventBatch.CbDeviceMetadata.newBuilder()
            .setOrgKey(orgKey)
            .setDeviceId(faker.number().numberBetween(9999900000L, 9999999999L))
            .setDeviceName(deviceName)
            .setDevicePolicy(policy)
            .setDeviceInstalledBy(faker.internet().username())
            .setSelfReportedDeviceIp(faker.internet().privateIpV4Address())
            .setBackendReportedDeviceIp(faker.internet().publicIpV4Address())
            .setDeviceOsType(os)
            .setDeviceTargetPriority(TARGET_PRIORITY_MEDIUM);

    if (os == WINDOWS) {
      device.setDeviceOsVersion(faker.computer().windows());
    } else if (os == PSCRAugmentedEventBatch.CbDeviceOsType.MAC) {
      device.setDeviceOsVersion(faker.computer().macos());
    } else if (os == PSCRAugmentedEventBatch.CbDeviceOsType.LINUX) {
      device.setDeviceOsVersion(faker.computer().linux());
      device.setDeviceTargetPriority(TARGET_PRIORITY_HIGH);
    }

    return device;
  }
}
