package com.vmware.psc.alerting.persister.integration.standards;

import static com.tngtech.archunit.lang.syntax.ArchRuleDefinition.methods;

import com.tngtech.archunit.core.domain.JavaMethod;
import com.tngtech.archunit.junit.AnalyzeClasses;
import com.tngtech.archunit.junit.ArchTest;
import com.tngtech.archunit.lang.ArchCondition;
import com.tngtech.archunit.lang.ArchRule;
import com.tngtech.archunit.lang.ConditionEvents;
import com.tngtech.archunit.lang.SimpleConditionEvent;
import org.junit.jupiter.params.ParameterizedTest;

@AnalyzeClasses(packages = "com.vmware.psc.alerting.persister.integration.test")
@SuppressWarnings("unused")
public class IntegrationTestCaseNamingTest {
  @ArchTest
  private static final ArchRule allIntegrationTestsHaveMeaningfulDescriptions =
      methods()
          .that()
          .areAnnotatedWith(ParameterizedTest.class)
          .should(new TestNameAnnotationCondition())
          .allowEmptyShould(true)
          .because("we want our test case names to be consistent and useful.");

  static class TestNameAnnotationCondition extends ArchCondition<JavaMethod> {
    public TestNameAnnotationCondition() {
      super("include a proper name");
    }

    @Override
    public void check(JavaMethod item, ConditionEvents events) {
      var annotation = item.getAnnotationOfType(ParameterizedTest.class);
      var testName = annotation.name();
      if (!testName.startsWith("{0}: ")) {
        var msg =
            String.format(
                "Test method %s must specify a test name that starts with \"{0}: \". (found: \"%s\")",
                item.getFullName(), testName);
        events.add(SimpleConditionEvent.violated(item, msg));
      }

      if (testName.length() < 15) {
        // This is an arbitrary value but should be long enough to weed out bad names
        var msg =
            String.format(
                "Test method %s must specify a test name that is at least 15 characters (found: \"%s\")",
                item.getFullName(), testName);
        events.add(SimpleConditionEvent.violated(item, msg));
      }
    }
  }
}
