package com.vmware.psc.alerting.persister.integration.test;

import com.vmware.psc.alerting.persister.integration.config.TestConfig;
import lombok.RequiredArgsConstructor;
import org.assertj.core.util.Strings;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.web.reactive.server.WebTestClient;

@SpringBootTest
@RequiredArgsConstructor(onConstructor_ = {@Autowired})
public class HealthCheckTest {
  private final TestConfig config;

  @Test
  void testHealthCheck() {
    var svcEndpoint = getKubernetesServiceEndpoint();
    var client = WebTestClient.bindToServer().baseUrl(svcEndpoint).build();
    client.get().uri("/alert-persister/health_check").exchange().expectStatus().is2xxSuccessful();
  }

  private String getKubernetesServiceEndpoint() {
    if (Strings.isNullOrEmpty(config.getServiceName())) {
      throw new IllegalArgumentException("Missing service name!");
    }

    if (config.getServicePort() == null || config.getServicePort() == 0) {
      throw new IllegalArgumentException("Missing service port!");
    }

    return String.format("http://%s:%s", config.getServiceName(), config.getServicePort());
  }
}
