package com.vmware.psc.alerting.persister;

import com.carbonblack.psc.orgkey.IOrgKeyService;
import com.carbonblack.psc.orgkey.OrgKeyService;
import java.util.Arrays;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;

@Slf4j
@SpringBootApplication
@EnableConfigurationProperties
public class AlertPersisterApplication {

  public static void main(String[] args) {
    ConfigurableApplicationContext app =
        SpringApplication.run(AlertPersisterApplication.class, args);

    List<String> profiles = Arrays.asList(app.getEnvironment().getActiveProfiles());

    if (!profiles.contains("aws") && !profiles.contains("localstack")) {
      log.error(
          "Invalid profiles activated. "
              + "Must specify either 'aws' or 'localstack' profile. "
              + "Check spring_profiles_active env var.");
      log.error("Current active profiles: {}", profiles);
      app.close();
    } else {
      log.info("Running with profiles: {}", profiles);
    }
  }

  @Bean
  IOrgKeyService orgKeyService() {
    return new OrgKeyService();
  }
}
