package com.vmware.psc.alerting.persister;

import com.carbonblack.protocols.psc.response.PSCAlert;
import com.vmware.psc.alerting.persister.cache.properties.CacheAlertAggProperties;
import com.vmware.psc.alerting.persister.cache.service.CacheService;
import com.vmware.psc.alerting.persister.es.AlertWriter;
import com.vmware.psc.alerting.persister.es.notes.NotesWriter;
import com.vmware.psc.alerting.persister.kinesis.producer.ProducerSupport;
import com.vmware.psc.alerting.persister.orgthreat.OrgThreatClient;
import com.vmware.psc.alerting.persister.util.RecordProcessingSupport;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;
import software.amazon.kinesis.processor.ShardRecordProcessorFactory;

@Slf4j
@Component
@Profile("(aws | localstack) & !poke")
public class PersisterRecordProcessorFactory implements ShardRecordProcessorFactory {

  private final AlertWriter alertWriter;
  private final NotesWriter notesWriter;
  private final OrgThreatClient orgThreatClient;

  private final CacheAlertAggProperties cacheAlertAggProperties;

  private final CacheService<String, PSCAlert.CbUaeAlert> alertCacheService;

  private final RecordProcessingSupport recordProcessingSupport;

  private final ProducerSupport producerSupport;

  public PersisterRecordProcessorFactory(
      final RecordProcessingSupport recordProcessingSupport,
      AlertWriter alertWriter,
      NotesWriter notesWriter,
      OrgThreatClient orgThreatClient,
      final CacheAlertAggProperties cacheAlertAggProperties,
      final CacheService<String, PSCAlert.CbUaeAlert> alertCacheService,
      final ProducerSupport producerSupport) {
    this.recordProcessingSupport = recordProcessingSupport;
    this.alertWriter = alertWriter;
    this.notesWriter = notesWriter;
    this.orgThreatClient = orgThreatClient;
    this.cacheAlertAggProperties = cacheAlertAggProperties;
    this.alertCacheService = alertCacheService;
    this.producerSupport = producerSupport;
  }

  @Override
  public PersisterRecordProcessor shardRecordProcessor() {
    log.debug("Alert Aggregation cache enabled {}", cacheAlertAggProperties.isEnabled());
    return new PersisterRecordProcessor(
        recordProcessingSupport,
        alertWriter,
        notesWriter,
        orgThreatClient,
        alertCacheService,
        cacheAlertAggProperties,
        producerSupport);
  }
}
