package com.vmware.psc.alerting.persister.aws;

import com.amazonaws.auth.AWSCredentialsProvider;
import com.amazonaws.auth.DefaultAWSCredentialsProviderChain;
import com.amazonaws.regions.AwsRegionProvider;
import com.amazonaws.regions.DefaultAwsRegionProviderChain;
import com.amazonaws.services.kinesis.producer.KinesisProducerConfiguration;
import com.vmware.psc.alerting.persister.properties.KinesisConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import software.amazon.awssdk.auth.credentials.*;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.cloudwatch.CloudWatchAsyncClient;
import software.amazon.awssdk.services.dynamodb.DynamoDbAsyncClient;
import software.amazon.awssdk.services.kinesis.KinesisAsyncClient;
import software.amazon.awssdk.services.kinesis.KinesisAsyncClientBuilder;

@Slf4j
@Configuration
@Profile("aws")
public class AwsConfig {

  @Bean
  public AWSCredentialsProvider awsCredentialsProvider() {
    return DefaultAWSCredentialsProviderChain.getInstance();
  }

  @Bean
  public AwsCredentialsProvider localstackCredentialsSdkV2() {
    return DefaultCredentialsProvider.create();
  }

  @Bean
  AwsRegionProvider regionProvider() {
    return new DefaultAwsRegionProviderChain();
  }

  @Bean
  public KinesisAsyncClient kinesisAsyncClient(
      AwsCredentialsProvider awsCredentialsProvider,
      AwsRegionProvider regionProvider,
      KinesisConfig config) {
    KinesisAsyncClientBuilder client =
        KinesisAsyncClient.builder()
            .credentialsProvider(awsCredentialsProvider)
            .region(Region.of(regionProvider.getRegion()));

    return client.build();
  }

  @Bean
  public DynamoDbAsyncClient buildAmazonDynamoDB(
      AwsCredentialsProvider awsCredentialsProvider, AwsRegionProvider regionProvider) {
    return DynamoDbAsyncClient.builder()
        .credentialsProvider(awsCredentialsProvider)
        .region(Region.of(regionProvider.getRegion()))
        .build();
  }

  @Bean
  public CloudWatchAsyncClient cloudWatch(
      AwsCredentialsProvider awsCredentialsProvider, AwsRegionProvider regionProvider) {
    return CloudWatchAsyncClient.builder()
        .credentialsProvider(awsCredentialsProvider)
        .region(Region.of(regionProvider.getRegion()))
        .build();
  }

  @Bean
  public KinesisProducerConfiguration kinesisProducerConfiguration(
      AWSCredentialsProvider awsCredentialsProvider, AwsRegionProvider regionProvider) {
    return new KinesisProducerConfiguration()
        .setCredentialsProvider(awsCredentialsProvider)
        .setRegion(regionProvider.getRegion());
  }
}
