package com.vmware.psc.alerting.persister.cache.listeners;

import com.carbonblack.protocols.psc.response.PSCAlert;
import com.vmware.psc.alerting.persister.enums.AggregationUpdateType;
import com.vmware.psc.alerting.persister.es.AlertWriter;
import com.vmware.psc.alerting.persister.util.CbUaeAlertUtil;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
import org.ehcache.event.CacheEvent;
import org.ehcache.event.CacheEventListener;

/**
 * Alert Cache Listener
 *
 * <p>This class listens for the EHCache Alert Event <strong>EVICT</strong> and persist in
 * OpenSearch
 */
@Slf4j
public class AlertCacheListener implements CacheEventListener<String, PSCAlert.CbUaeAlert> {

  private final AlertWriter alertWriter;

  public AlertCacheListener() {
    this.alertWriter = SpringContext.getBean(AlertWriter.class);
  }

  @Override
  public void onEvent(final CacheEvent<? extends String, ? extends PSCAlert.CbUaeAlert> event) {
    if (event != null
        && event.getOldValue() != null
        && event.getOldValue() instanceof PSCAlert.CbUaeAlert) {
      final PSCAlert.CbUaeAlert cbUaeAlert = event.getOldValue();
      if (CbUaeAlertUtil.getUpdateCount(cbUaeAlert) > 0) {
        try {
          log.debug(
              "Ehcache event listener triggered and alert id={}",
              cbUaeAlert.getAlert().getAlertIdGuid());
          final PSCAlert.CbUaeAlert.Builder builder = PSCAlert.CbUaeAlert.newBuilder(cbUaeAlert);
          final List<PSCAlert.CbUaeAlert.Builder> alertBuilderList = new ArrayList<>();
          alertBuilderList.add(builder);
          alertWriter.writeAndClearAlerts(alertBuilderList, AggregationUpdateType.EVICT);
          log.debug("Written evicted alert to ES cache");
        } catch (final IOException ioException) {
          throw new IllegalStateException(ioException);
        }
      }
    }
  }
}
