package com.vmware.psc.alerting.persister.cache.metrics;

import static com.google.common.base.Preconditions.checkArgument;

import com.vmware.psc.alerting.persister.enums.AggregationUpdateType;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Tags;
import java.util.EnumMap;
import java.util.Map;

public class CacheAlertAggregationMetrics {

  private final Map<AggregationUpdateType, Counter> counters;

  public CacheAlertAggregationMetrics(MeterRegistry meterRegistry, String baseName) {
    checkArgument(meterRegistry != null);
    counters = new EnumMap<>(AggregationUpdateType.class);

    for (final AggregationUpdateType aggregationUpdateType : AggregationUpdateType.values()) {
      counters.put(
          aggregationUpdateType,
          meterRegistry.counter(
              baseName, Tags.of("aggregation_type", aggregationUpdateType.name())));
    }
  }

  public void increment(AggregationUpdateType aggregationUpdateType, int count) {
    counters.get(aggregationUpdateType).increment(count);
  }
}
