package com.vmware.psc.alerting.persister.es;

import com.carbonblack.protocols.psc.response.PSCAlert;
import com.google.protobuf.util.JsonFormat;
import java.io.IOException;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import lombok.extern.slf4j.Slf4j;
import org.opensearch.OpenSearchStatusException;
import org.opensearch.action.search.SearchRequest;
import org.opensearch.action.search.SearchResponse;
import org.opensearch.client.RequestOptions;
import org.opensearch.client.RestHighLevelClient;
import org.opensearch.common.unit.TimeValue;
import org.opensearch.index.query.QueryBuilders;
import org.opensearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Component;

/** AlertClient */
@Component
@Slf4j
public class AlertEsClient {

  private final RestHighLevelClient esClient;

  /**
   * Constructor injection.
   *
   * @param esClusterMap Map.
   */
  public AlertEsClient(final Map<String, RestHighLevelClient> esClusterMap) {
    this.esClient = Optional.of(esClusterMap).map(map -> map.get("alerts")).orElseThrow();
  }

  /**
   * Search alert by id.
   *
   * @param id String.
   * @return CbUaeAlert
   */
  public Optional<PSCAlert.CbUaeAlert> findAlertById(String id) throws IOException {
    SearchResponse results;
    try {
      results =
          esClient.search(
              new SearchRequest()
                  .indices("alert-*")
                  .source(
                      new SearchSourceBuilder()
                          .query(QueryBuilders.queryStringQuery("alert.alertIdGuid:\"" + id + "\""))
                          .from(0)
                          .size(1)
                          .timeout(new TimeValue(5, TimeUnit.SECONDS))),
              RequestOptions.DEFAULT);
    } catch (OpenSearchStatusException | IOException ex) {
      log.error("getAlertById: Error during query for alert {}", id, ex);
      return Optional.empty();
    }

    if (results.getHits().getHits().length != 1) {
      log.debug("getAlertById: Got {} hits for alert {}", results.getHits().getHits().length, id);
      return Optional.empty();
    }

    log.debug("getAlertById: found alert {}", id);
    PSCAlert.CbUaeAlert.Builder builder = PSCAlert.CbUaeAlert.newBuilder();
    JsonFormat.parser().merge(results.getHits().getHits()[0].getSourceAsString(), builder);
    return Optional.of(builder.build());
  }
}
