package com.vmware.psc.alerting.persister.es.notes;

import com.vmware.psc.alerting.persister.es.IPartitionStrategyV2;
import com.vmware.psc.alerting.persister.model.notes.Notes;
import com.vmware.psc.alerting.persister.properties.NotesConfig;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

@Component
@AllArgsConstructor
public class NotesPartitionStrategy {

  private final IPartitionStrategyV2 monthlyFixedRetentionPartitionStrategy;
  private final IPartitionStrategyV2 yearlyFixedRetentionPartitionStrategy;
  private final NotesConfig notesConfig;

  /**
   * Get index for notes based on the field. If alertId is set, it should go to notes-alert-*. If
   * threatId is set, it should go to notes-threat-* notes-suppression-* has been deprecated and
   * moved to alert notes.
   */
  public String getIndexForNotes(Notes notes) {
    if (notes.getAlertId() != null) {
      return monthlyFixedRetentionPartitionStrategy.getIndexForDocument(
          notesConfig.getAlertLevelIndexPrefix(), notes.getCreateTimestamp());
    }
    return yearlyFixedRetentionPartitionStrategy.getIndexForDocument(
        notesConfig.getThreatLevelIndexPrefix(), notes.getCreateTimestamp());
  }
}
