package com.vmware.psc.alerting.persister.kinesis.client;

import static com.google.common.base.Preconditions.checkArgument;

import com.vmware.psc.alerting.persister.kinesis.utils.MeterName;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.DistributionSummary;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Timer;

class RecordProcessorMetrics {
  /** Histogram for record batch size (number of records). */
  DistributionSummary recordBatchSize;
  /** Histogram for record batch size in bytes (size of all records). */
  DistributionSummary recordBatchBytes;
  /** Histogram for size of a single record in bytes. */
  DistributionSummary recordBytes;
  /** Meter for how far we are behind the latest record. (bucketed at the instance level) */
  DistributionSummary latencyMs;
  /** Time to process a record batch. */
  Timer recordBatchProcessingTime;
  /** Time to parse a record batch. */
  Timer recordBatchParsingTime;
  /** Time to process a single record. */
  Timer recordProcessingTime;
  /** Average checkpoint time. */
  Timer checkpointTime;
  /** Number of records that had a parsing error. */
  Counter recordParsingErrors;
  /** Number of records that had to be retried. */
  Counter recordProcessingRetries;
  /** Number of records that had a processing error. */
  Counter recordProcessingErrors;
  /** Number of times checkpoint was throttled. */
  Counter checkpointRetryCount;
  /** Number of times checkpoint failed. */
  Counter checkpointErrorCount;

  RecordProcessorMetrics(MeterRegistry registry, String prefix) {
    checkArgument(registry != null);

    final String metricPrefix = MeterName.join("STREAMING", prefix);

    recordBatchSize = registry.summary(MeterName.join(metricPrefix, "record_batch_size"));
    recordBatchBytes = registry.summary(MeterName.join(metricPrefix, "record_batch_bytes"));
    recordBytes = registry.summary(MeterName.join(metricPrefix, "record_bytes"));
    latencyMs = registry.summary(MeterName.join(metricPrefix, "latency_ms"));

    recordBatchProcessingTime =
        registry.timer(MeterName.join(metricPrefix, "record_batch_processing_time"));
    recordBatchParsingTime =
        registry.timer(MeterName.join(metricPrefix, "record_batch_parsing_time"));
    recordProcessingTime = registry.timer(MeterName.join(metricPrefix, "record_processing_time"));
    checkpointTime = registry.timer(MeterName.join(metricPrefix, "checkpoint_time"));

    recordParsingErrors = registry.counter(MeterName.join(metricPrefix, "record_parsing_errors"));
    recordProcessingErrors =
        registry.counter(MeterName.join(metricPrefix, "record_processing_errors"));
    recordProcessingRetries =
        registry.counter(MeterName.join(metricPrefix, "record_processing_retries"));
    checkpointRetryCount = registry.counter(MeterName.join(metricPrefix, "checkpoint_retry_count"));
    checkpointErrorCount = registry.counter(MeterName.join(metricPrefix, "checkpoint_error_count"));
  }
}
