package com.vmware.psc.alerting.persister.kinesis.client.health;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.actuate.health.Health;
import org.springframework.boot.actuate.health.HealthIndicator;
import org.springframework.boot.actuate.health.Status;
import org.springframework.stereotype.Component;
import software.amazon.kinesis.coordinator.WorkerStateChangeListener;

@Component
@RequiredArgsConstructor
@Slf4j
public class KinesisApplicationHealthIndicator
    implements HealthIndicator, WorkerStateChangeListener {

  private WorkerStateChangeListener.WorkerState currentState;

  @Override
  public Health health() {
    Status status = currentState == WorkerState.STARTED ? Status.UP : Status.DOWN;
    return Health.status(status).withDetail("worker_state", currentState).build();
  }

  @Override
  public void onWorkerStateChange(WorkerState newState) {
    log.info("Kinesis worker state changed to {}", newState);
    currentState = newState;
  }

  @Override
  public void onAllInitializationAttemptsFailed(Throwable e) {
    log.error("Kinesis worker initialization failed!", e);
    currentState = null;
  }
}
