package com.vmware.psc.alerting.persister.kinesis.producer.properties;

import com.amazonaws.services.kinesis.producer.KinesisProducerConfiguration;
import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
public class KinesisKplProducerProperties {
  boolean enabled = false;

  /**
   * Maximum amount of time (milliseconds) a record may spend being buffered before it gets sent.
   *
   * <p>Records may be sent sooner than this depending on the other buffering limits.
   *
   * <p>For more detail, see {@link KinesisProducerConfiguration#setRecordMaxBufferedTime(long)}
   */
  long maxQueueTimeMs = 500;

  /**
   * Maximum number of entries allowed to be queued at one time.
   *
   * <p>When the queue is full, the producer enters a sleep loop to wait for more capacity to become
   * available.
   */
  long maxQueueSize = 500;

  /**
   * Controls the number of metrics that are uploaded to CloudWatch.
   *
   * <p>Allowed values: "none", "summary", or "detailed"
   *
   * <p>For more detail, see {@link KinesisProducerConfiguration#setMetricsLevel(String)}
   */
  String metricsLevel = "none";

  /**
   * Limits the maximum allowed put rate for a shard, as a percentage of the backend limits.
   *
   * <p>For more detail, see {@link KinesisProducerConfiguration#setRateLimit(long)}
   */
  long rateLimit = 120;

  /**
   * Maximum number of connections to open to the backend. HTTP requests are sent in parallel over
   * multiple connections.
   *
   * <p>For more detail, see {@link KinesisProducerConfiguration#setMaxConnections(long)}
   */
  long maxConnections = 24;

  /**
   * Set a time-to-live on records (milliseconds). Records that do not get successfully put within
   * the limit are failed.
   *
   * <p>For more detail, see {@link KinesisProducerConfiguration#setRecordTtl(long)}
   */
  long recordTtl = 86400000;

  /**
   * Sets the maximum number of threads that the native process' thread pool will be configured
   * with.
   *
   * <p>For more detail, see {@link KinesisProducerConfiguration#setThreadPoolSize(int)}
   */
  int threadPoolSize = 0;
}
