package com.vmware.psc.alerting.persister.kinesis.utils;

import com.google.protobuf.InvalidProtocolBufferException;
import com.google.protobuf.Message;
import com.google.protobuf.MessageOrBuilder;
import com.google.protobuf.util.JsonFormat;

/** Deserializes json to protobuf object. */
public final class ProtoUtilHelper {
  /**
   * Sample use: PSCAlert.CbUaeAlert cbAlert = ProtoUtil.toProto(esCbUaeJson,
   * PSCAlert.CbUaeAlert.getDefaultInstance());
   *
   * @param protoJson The json string to be converted to protobuf format
   * @param message The protobuf object to be converted
   * @param <T> Is of type protobuf
   * @return The protobuf object converted from its json representation
   */
  public static <T extends MessageOrBuilder> T toProto(String protoJson, T message)
      throws InvalidProtocolBufferException {
    Message.Builder builder = message.getDefaultInstanceForType().toBuilder();
    JsonFormat.parser().ignoringUnknownFields().merge(protoJson, builder);
    T out = (T) builder.build();
    return out;
  }

  /**
   * Gets the JSON string representation of the protobuf
   *
   * @param proto The protobuf object
   * @param <T> extends {@link MessageOrBuilder}
   * @return The JSON string of the protobuf object
   * @throws InvalidProtocolBufferException
   */
  public static <T extends MessageOrBuilder> String toString(T proto)
      throws InvalidProtocolBufferException {
    return JsonFormat.printer()
        .includingDefaultValueFields()
        .omittingInsignificantWhitespace()
        .print(proto);
  }
}
