package com.vmware.psc.alerting.persister.metrics;

import com.amazonaws.regions.AwsRegionProvider;
import com.google.common.collect.ImmutableList;
import io.micrometer.core.aop.TimedAspect;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Tag;
import org.springframework.boot.actuate.autoconfigure.metrics.MeterRegistryCustomizer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;

@Configuration
@Profile({"aws", "localstack"})
public class MetricsConfig {

  /** enables the {@link io.micrometer.core.annotation.Timed} annotation on arbitrary methods */
  @Bean
  public TimedAspect timedAspect(MeterRegistry registry) {
    return new TimedAspect(registry);
  }

  @Bean
  MeterRegistryCustomizer<MeterRegistry> metricsCommonTags(final AwsRegionProvider regionProvider) {

    return registry ->
        registry
            .config()
            .commonTags(
                ImmutableList.of(
                    Tag.of("service", "alert-persister"),
                    Tag.of("region", regionProvider.getRegion())));
  }
}
