package com.vmware.psc.alerting.persister.aws.localstack;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.hasItem;

import com.vmware.psc.alerting.persister.AlertPersisterSpringBootTestWithMockKinesisProducer;
import com.vmware.psc.alerting.persister.properties.KinesisConfig;
import java.util.*;
import java.util.concurrent.ExecutionException;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import software.amazon.awssdk.services.dynamodb.DynamoDbAsyncClient;
import software.amazon.awssdk.services.dynamodb.model.ListTablesResponse;
import software.amazon.awssdk.services.kinesis.KinesisAsyncClient;
import software.amazon.awssdk.services.kinesis.model.ListStreamsResponse;

public class LocalstackInfraTest extends AlertPersisterSpringBootTestWithMockKinesisProducer {

  @Autowired KinesisAsyncClient kinesis;

  @Autowired DynamoDbAsyncClient dynamodb;

  @Autowired KinesisConfig config;

  @Test
  void testKinesisStreamsExist() throws ExecutionException, InterruptedException {
    ListStreamsResponse streamsResult = kinesis.listStreams().get();
    List<String> streams = streamsResult.streamNames();

    assertThat(streams, hasItem(config.getClient().getStreamName()));
    assertThat(streams, hasItem(config.getOutputStream().getStreamName()));
  }

  @Test
  void testDynamoTablesExist() throws ExecutionException, InterruptedException {
    ListTablesResponse tablesResult = dynamodb.listTables().get();
    List<String> tables = tablesResult.tableNames();

    assertThat(tables, hasItem(config.getClient().getApplicationName()));
  }
}
