package com.vmware.psc.alerting.persister.es;

import static org.awaitility.Awaitility.await;
import static org.junit.jupiter.api.Assertions.*;

import com.carbonblack.protocols.psc.response.PSCAlert;
import com.carbonblack.protocols.psc.response.PSCRAugmentedEventBatch;
import com.vmware.psc.alerting.data.uae.UaeAlertCreator;
import com.vmware.psc.alerting.exception.NonRetryableDecompressionException;
import com.vmware.psc.alerting.persister.BaseAlertPersisterSpringBootTest;
import com.vmware.psc.alerting.persister.enums.AggregationUpdateType;
import com.vmware.psc.alerting.sampledata.UdsInputSamples;
import com.vmware.time.helper.TimeHelper;
import java.io.IOException;
import java.time.*;
import java.util.List;
import java.util.Objects;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

class AlertEsClientTest extends BaseAlertPersisterSpringBootTest {

  @Autowired AlertEsClient alertEsClient;

  @Autowired AlertWriter writer;

  @Test
  void findAlertByIdWithDocExist() throws IOException, NonRetryableDecompressionException {
    PSCRAugmentedEventBatch.CbAugmentedEventBatchMsg batch =
        UdsInputSamples.deviceControlEvent()
            .withTriggeredAlertId("some-alert-" + System.currentTimeMillis())
            .withAlertCreationTimestamp(TimeHelper.timestampNow())
            .build();

    List<PSCAlert.CbUaeAlert> alerts = UaeAlertCreator.convertAlerts(batch);
    writer.writeAlertBatch(alerts, AggregationUpdateType.NEW);
    final String id =
        alerts.stream().findFirst().map(alert -> alert.getAlert().getAlertIdGuid()).orElse(null);
    PSCAlert.CbUaeAlert response =
        await()
            .atMost(Duration.ofSeconds(60))
            .until(() -> alertEsClient.findAlertById(id).orElse(null), Objects::nonNull);
    assertNotNull(response);
  }

  @Test
  void findAlertByIdWithDocNotExist() throws IOException {
    final PSCAlert.CbUaeAlert response = alertEsClient.findAlertById("test").orElse(null);
    assertNull(response);
  }
}
