package com.vmware.psc.alerting.persister.es;

import static org.junit.jupiter.api.Assertions.assertEquals;

import com.carbonblack.protocols.psc.response.PSCAlert;
import com.carbonblack.protocols.psc.response.PSCRProto;
import com.google.protobuf.Timestamp;
import com.vmware.psc.alerting.data.uae.UaeAlertCreator;
import com.vmware.psc.alerting.exception.NonRetryableDecompressionException;
import com.vmware.psc.alerting.sampledata.UdsInputSampleBuilder;
import com.vmware.psc.alerting.sampledata.UdsInputSamples;
import com.vmware.time.helper.TimeHelper;
import java.io.IOException;
import java.time.Instant;
import java.util.List;
import org.junit.jupiter.api.Test;

public class DailyFixedRetentionPartitionStrategyTest {
  IPartitionStrategy strategy = new DailyFixedRetentionPartitionStrategy();

  private static final Instant INGRESS_TIME = Instant.parse("2021-01-01T00:00:00.000Z");
  private static final Timestamp INGRESS_TIMESTAMP = TimeHelper.toTimestamp(INGRESS_TIME);
  private static final Instant CREATE_TIME = Instant.parse("2021-01-02T00:00:00.000Z");
  private static final Timestamp CREATE_TIMESTAMP = TimeHelper.toTimestamp(CREATE_TIME);

  // Helpers - known expected values for each partition type
  private static final String INGRESS_TIME_PARTITION = "alert-2021-01-01-r180";
  private static final String CREATE_TIME_PARTITION = "alert-2021-01-02-r180";

  @Test
  void testGetWriteIndexHashbanAlert() throws IOException, NonRetryableDecompressionException {
    var alert = getAlert(UdsInputSamples.hashbanDenyEvent());
    assertEquals(INGRESS_TIME_PARTITION, strategy.getIndexForDocument(alert));
  }

  @Test
  void testGetWriteIndexAnalyticsSecurityEvent()
      throws IOException, NonRetryableDecompressionException {
    var alert = getAlert(UdsInputSamples.mappedThreatProcessEventWithAttackTactic());
    assertEquals(CREATE_TIME_PARTITION, strategy.getIndexForDocument(alert));
  }

  @Test
  void testGetWriteIndexDreSecurityEvent() throws IOException, NonRetryableDecompressionException {
    var alert = getAlert(UdsInputSamples.threatEventWithAttackTactic());
    assertEquals(INGRESS_TIME_PARTITION, strategy.getIndexForDocument(alert));
  }

  @Test
  void testGetWriteIndexContainerRuntimeAlert()
      throws IOException, NonRetryableDecompressionException {
    var alert = getAlert(UdsInputSamples.containerRuntimeEvent());
    assertEquals(CREATE_TIME_PARTITION, strategy.getIndexForDocument(alert));
  }

  @Test
  void testGetWriteAggregatedAlert() throws IOException, NonRetryableDecompressionException {
    var alert =
        getAlert(
            UdsInputSamples.hostBasedFirewallEvent()
                .withAlertAggregation(
                    PSCRProto.CbAlertAggregation.newBuilder()
                        .setIsAggregated(true)
                        .setAggregateGuid("some-agg-guid")));
    assertEquals(INGRESS_TIME_PARTITION, strategy.getIndexForDocument(alert));
  }

  private PSCAlert.CbUaeAlert getAlert(UdsInputSampleBuilder builder)
      throws NonRetryableDecompressionException, IOException {
    List<PSCAlert.CbUaeAlert> alerts =
        UaeAlertCreator.convertAlerts(
            builder
                .backdatedTo(CREATE_TIME)
                .withAlertCreationTimestamp(CREATE_TIMESTAMP)
                .withIngressTimestamp(INGRESS_TIMESTAMP)
                .build());
    return alerts.get(0);
  }
}
