package com.vmware.psc.alerting.persister.es;

import static org.junit.jupiter.api.Assertions.assertEquals;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.vmware.psc.alerting.persister.AlertPersisterSpringBootTestWithMockKinesisProducer;
import com.vmware.psc.alerting.persister.es.notes.NotesPartitionStrategy;
import com.vmware.psc.alerting.persister.es.notes.NotesWriter;
import com.vmware.psc.alerting.persister.model.notes.Notes;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.opensearch.action.admin.indices.delete.DeleteIndexRequest;
import org.opensearch.action.admin.indices.refresh.RefreshRequest;
import org.opensearch.action.get.GetRequest;
import org.opensearch.action.get.GetResponse;
import org.opensearch.client.RequestOptions;
import org.opensearch.client.RestHighLevelClient;
import org.springframework.beans.factory.annotation.Autowired;

public class NotesWriterTest extends AlertPersisterSpringBootTestWithMockKinesisProducer {
  @Autowired private NotesWriter writer;
  @Autowired private NotesPartitionStrategy notesPartitionStrategy;
  @Autowired Map<String, RestHighLevelClient> esClusterMap;
  @Autowired private ObjectMapper objectMapper;

  private RestHighLevelClient client;
  private String currentUsedIndex;

  @BeforeAll
  public void before() {
    client = esClusterMap.get("notes");
  }

  @AfterAll
  void cleanIndices() throws IOException {
    client.indices().delete(new DeleteIndexRequest(currentUsedIndex), RequestOptions.DEFAULT);
  }

  @Test
  public void testWriter() throws IOException {
    String suppressionRuleId = "sup-id-1";
    long version = 1;
    boolean isInternal = true;
    String alertId = UUID.randomUUID().toString();
    String id = NotesWriter.buildNotesId(alertId, suppressionRuleId, version, isInternal);
    String source = "suppression";
    String content = "blahblah";
    String orgKey = "local";
    String author = "mdr@vmware.com";
    Notes note1 =
        Notes.builder()
            .alertId(alertId)
            .suppressionRuleId(suppressionRuleId)
            .internal(isInternal)
            .source(source)
            .orgKey(orgKey)
            .note(content)
            .author(author)
            .id(id)
            .build();

    suppressionRuleId = "sup-id-2";
    String id2 = NotesWriter.buildNotesId(alertId, suppressionRuleId, version, isInternal);
    Notes note2 =
        Notes.builder()
            .alertId(alertId)
            .suppressionRuleId(suppressionRuleId)
            .internal(isInternal)
            .source(source)
            .orgKey(orgKey)
            .note(content)
            .author(author)
            .id(id2)
            .build();

    suppressionRuleId = "sup-id-3";
    String id3 = NotesWriter.buildNotesId(alertId, suppressionRuleId, version, isInternal);
    Notes note3 =
        Notes.builder()
            .alertId(alertId)
            .suppressionRuleId(suppressionRuleId)
            .internal(isInternal)
            .source(source)
            .orgKey(orgKey)
            .note(content)
            .author(author)
            .id(id3)
            .build();
    List<Notes> flushList = new ArrayList<>();
    flushList.add(note1);
    flushList.add(note2);
    flushList.add(note3);

    writer.writeNotesBatch(flushList);
    refresh();

    Notes notes1FromEs = getNoteById(note1);
    assertEquals(source, notes1FromEs.getSource());

    Notes notes2FromEs = getNoteById(note2);
    assertEquals(source, notes2FromEs.getSource());

    Notes notes3FromEs = getNoteById(note3);
    assertEquals(source, notes3FromEs.getSource());
  }

  private Notes getNoteById(Notes notes) throws IOException {
    currentUsedIndex = notesPartitionStrategy.getIndexForNotes(notes);
    GetResponse doc =
        client.get(new GetRequest(currentUsedIndex, notes.getId()), RequestOptions.DEFAULT);
    return objectMapper.readValue(doc.getSourceAsString(), Notes.class);
  }

  private void refresh() throws IOException {
    RefreshRequest requestAll = new RefreshRequest();
    client.indices().refresh(requestAll, RequestOptions.DEFAULT);
  }
}
