package com.vmware.psc.alerting.persister.kinesis.integration;

import static org.awaitility.Awaitility.await;

import com.vmware.psc.alerting.persister.aws.localstack.LocalstackInfra;
import com.vmware.psc.alerting.persister.kinesis.client.KinesisApplication;
import com.vmware.psc.alerting.persister.kinesis.client.properties.KinesisClientProperties;
import java.time.Duration;
import software.amazon.awssdk.services.cloudwatch.CloudWatchAsyncClient;
import software.amazon.awssdk.services.dynamodb.DynamoDbAsyncClient;
import software.amazon.awssdk.services.kinesis.KinesisAsyncClient;
import software.amazon.kinesis.processor.ShardRecordProcessorFactory;

public class LocalKinesisApplication extends KinesisApplication {
  private final LocalstackInfra infra;

  public LocalKinesisApplication(
      LocalstackInfra infra,
      KinesisClientProperties properties,
      KinesisAsyncClient kinesis,
      DynamoDbAsyncClient dynamodb,
      CloudWatchAsyncClient cloudwatch,
      ShardRecordProcessorFactory recordProcessorFactory) {
    super(properties, kinesis, dynamodb, cloudwatch, recordProcessorFactory);
    this.infra = infra;
  }

  @Override
  public void afterPropertiesSet() throws Exception {
    infra.createStreamIfNotExists(properties.getStreamName());
    infra.createTableIfNotExists(properties.getApplicationName());
    await()
        .atMost(Duration.ofSeconds(60))
        .until(() -> infra.streamExists(properties.getStreamName()));
    await()
        .atMost(Duration.ofSeconds(60))
        .until(() -> infra.tableExists(properties.getApplicationName()));
    super.afterPropertiesSet();
  }

  @Override
  public void destroy() throws Exception {
    super.destroy();
    infra.deleteStream(properties.getStreamName());
    infra.deleteTable(properties.getApplicationName());
  }
}
