package com.vmware.psc.alerting.persister.kinesis.integration;

import com.carbonblack.psc.backoff.ExponentialBackoff;
import com.vmware.psc.alerting.persister.kinesis.client.RecordProcessorImpl;
import com.vmware.psc.alerting.persister.kinesis.client.properties.RecordProcessorProperties;
import io.micrometer.core.instrument.MeterRegistry;
import java.util.ArrayList;
import java.util.List;
import lombok.Getter;

public class SimpleStringKinesisConsumer extends RecordProcessorImpl<String> {

  @Getter private final List<String> received = new ArrayList<>();
  @Getter private final List<String> toCheckpoint = new ArrayList<>();
  @Getter private final List<String> checkpointed = new ArrayList<>();

  public SimpleStringKinesisConsumer(
      MeterRegistry registry,
      RecordProcessorProperties processorProperties,
      ExponentialBackoff backoff) {
    super(registry, processorProperties, backoff, backoff);
  }

  @Override
  protected String deserialize(byte[] serializedData) {
    return new String(serializedData);
  }

  @Override
  protected boolean process(String data) {
    received.add(data);
    toCheckpoint.add(data);
    return true;
  }

  @Override
  protected void beforeCheckpoint() {
    checkpointed.addAll(toCheckpoint);
    toCheckpoint.clear();
  }
}
