package com.vmware.psc.alerting.persister.kinesis.integration;

import com.carbonblack.psc.backoff.ExponentialBackoff;
import com.vmware.psc.alerting.persister.kinesis.client.properties.RecordProcessorProperties;
import io.micrometer.core.instrument.MeterRegistry;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import software.amazon.kinesis.processor.ShardRecordProcessor;
import software.amazon.kinesis.processor.ShardRecordProcessorFactory;

public class SimpleStringKinesisConsumerFactory implements ShardRecordProcessorFactory {
  private final List<SimpleStringKinesisConsumer> consumers;
  private final MeterRegistry registry;
  private final RecordProcessorProperties processorProperties;
  private final ExponentialBackoff backoff;

  public SimpleStringKinesisConsumerFactory(
      MeterRegistry registry,
      RecordProcessorProperties processorProperties,
      ExponentialBackoff backoff) {
    consumers = new ArrayList<>();
    this.registry = registry;
    this.processorProperties = processorProperties;
    this.backoff = backoff;
  }

  @Override
  public ShardRecordProcessor shardRecordProcessor() {
    SimpleStringKinesisConsumer consumer =
        new SimpleStringKinesisConsumer(registry, processorProperties, backoff);
    consumers.add(consumer);
    return consumer;
  }

  public List<String> allReceivedData() {
    return consumers.stream()
        .map(SimpleStringKinesisConsumer::getReceived)
        .flatMap(List::stream)
        .collect(Collectors.toList());
  }
}
