package com.vmware.psc.alerting.persister.orgthreat;

import static org.awaitility.Awaitility.await;
import static org.junit.jupiter.api.Assertions.*;

import com.carbonblack.psc.orgkey.IOrgKeyService;
import com.vmware.psc.alerting.persister.AlertPersisterSpringBootTestWithMockKinesisProducer;
import com.vmware.psc.alerting.persister.exception.RetryableOrgThreatException;
import java.io.IOException;
import java.time.Duration;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.cache.interceptor.SimpleKey;
import software.amazon.awssdk.services.dynamodb.DynamoDbAsyncClient;

class OrgThreatClientTest extends AlertPersisterSpringBootTestWithMockKinesisProducer {

  @Autowired DynamoDbAsyncClient dynamo;
  @Autowired IOrgKeyService orgKeyService;
  @Autowired OrgThreatClient client;
  @Autowired CacheManager cacheManager;

  @Test
  void writeOrgThreat() throws IOException, RetryableOrgThreatException {
    OrgThreat orgThreat =
        OrgThreat.builder()
            .orgId(555)
            .version(0)
            .threatId("some-threat")
            .workFlow(
                ThreatWorkflow.builder()
                    .changedBy("someone@vmware.com")
                    .comment("no comment.")
                    .lastUpdatedTime(System.currentTimeMillis())
                    .state(WorkflowState.OPEN)
                    .build())
            .build();

    String orgKey = orgKeyService.encodeOrgKey("unittest", 555);

    client.writeOrgThreat(orgKey, orgThreat);

    OrgThreat loaded = client.getById(orgKey, orgThreat.getThreatId());
    assertEquals(orgThreat, loaded);

    loaded.getWorkFlow().setComment("new comment");
    client.writeOrgThreat(orgKey, loaded);

    Cache cache = cacheManager.getCache("orgthreat");
    assertNotNull(cache, "Expected to find a cache for orgthreat");
    SimpleKey cacheKey = new SimpleKey(orgKey, orgThreat.getThreatId());
    OrgThreat cached = cache.get(cacheKey, OrgThreat.class);
    assertEquals(orgThreat, cached);

    await()
        .atMost(Duration.ofSeconds(45))
        .until(() -> client.getById(orgKey, orgThreat.getThreatId()), loaded::equals);
  }
}
