package com.vmware.psc.alerting.persister.properties;

import static org.junit.jupiter.api.Assertions.*;

import com.vmware.psc.alerting.persister.kinesis.client.properties.KinesisClientProperties;
import java.time.Instant;
import java.util.Date;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import software.amazon.kinesis.common.InitialPositionInStream;

public class KinesisClientPropertiesTest {

  private KinesisClientProperties properties;

  @BeforeEach
  void setup() {
    properties = new KinesisClientProperties();
    properties.setStreamName("any-stream");
    properties.setApplicationName("any-app-name");
  }

  @Test
  void testDefaultConfigValidWithStreamAndApplicationSet() {
    assertDoesNotThrow(properties::validate);
  }

  @Test
  void testValidationMissingApplicationName() {
    assertDoesNotThrow(properties::validate);
    properties.setApplicationName(null);
    assertThrows(IllegalArgumentException.class, properties::validate);
    properties.setApplicationName("");
    assertThrows(IllegalArgumentException.class, properties::validate);
  }

  @Test
  void testValidationMissingStreamName() {
    assertDoesNotThrow(properties::validate);
    properties.setStreamName(null);
    assertThrows(IllegalArgumentException.class, properties::validate);
    properties.setStreamName("");
    assertThrows(IllegalArgumentException.class, properties::validate);
  }

  @Test
  void testValidationMissingDateWhenAtTimestampSpecified() {
    assertDoesNotThrow(properties::validate);
    properties.setInitialPositionInStream(InitialPositionInStream.AT_TIMESTAMP);
    properties.setInitialPositionInStreamTimestamp(Date.from(Instant.now()));
    assertDoesNotThrow(properties::validate);
    properties.setInitialPositionInStreamTimestamp(null);
    assertThrows(NullPointerException.class, properties::validate);
  }
}
