package com.vmware.psc.alerting.persister.properties.converter;

import static org.junit.jupiter.api.Assertions.*;

import com.vmware.psc.alerting.persister.kinesis.producer.IKinesisProducer;
import com.vmware.psc.alerting.persister.properties.KinesisConfig;
import java.text.ParseException;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

@SpringBootTest(
    properties = {
      "alert-persister.kinesis.client.initialPositionInStreamTimestamp=2020-02-02 20:20:22",
    })
public class StringToDateConverterTest {
  @Autowired StringToDateConverter converter;

  @Autowired KinesisConfig config;

  @MockBean IKinesisProducer kinesisProducer;

  @Test
  void testPropertyInjected() {
    assertEquals(
        1580674822000L, config.getClient().getInitialPositionInStreamTimestamp().getTime());
  }

  @Test
  void testInvalidThrows() {
    assertThrows(ParseException.class, () -> converter.convert("this isn't a date sillyhead"));
  }

  @Test
  void testEmptyReturnsNull() {
    assertNull(converter.convert(""));
    assertNull(converter.convert(null));
  }
}
